@@ -19,7 +19,6 @@
 #include "common/platform.h"
 
 #include "common/access_control_list.h"
-#include "common/posix_acl_xattr.h"
 
 namespace aclConverter {
 
@@ -29,19 +28,14 @@ namespace aclConverter {
 	LIZARDFS_CREATE_EXCEPTION_CLASS(AclConversionException, Exception);
 
 	/*
-	 * An exception of failure during extraction xattr to POSIX object
+	 * An exception of failure during extraction xattr to ACL object
 	 */
 	LIZARDFS_CREATE_EXCEPTION_CLASS(PosixExtractionException, Exception);
 
 	/*
 	 * Get POSIX ACL object from xattr value
 	 */
-	PosixAclXattr extractPosixObject(const uint8_t* buffer, uint32_t bufferSize);
-
-	/*
-	 * Generate AccessControlList object from POSIX ACL xattr object
-	 */
-	AccessControlList posixToAclObject(const PosixAclXattr& posix);
+	AccessControlList extractAclObject(const uint8_t* buffer, uint32_t bufferSize);
 
 	/*
 	 * Generate xattr value from AccessControlList object
