@@ -22,6 +22,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <grp.h>
+#include <ios>
 #include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
@@ -46,6 +47,7 @@
 #include "common/cwrap.h"
 #include "common/exceptions.h"
 #include "common/exit_status.h"
+#include "common/event_loop.h"
 #include "common/main.h"
 #include "common/massert.h"
 #include "common/mfserr.h"
@@ -92,459 +94,48 @@ enum class RunMode {
 	kIsAlive = 6
 };
 
-enum class ExitingStatus {
-	kRunning = 0,
-	kWantExit = 1,
-	kCanExit = 2,
-	kDoExit = 3
-};
-
-static bool nextPollNonblocking = false;
-
-typedef struct deentry {
-	void (*fun)(void);
-	struct deentry *next;
-} deentry;
-
-static deentry *dehead=NULL;
-
-
-typedef struct weentry {
-	void (*fun)(void);
-	struct weentry *next;
-} weentry;
-
-static weentry *wehead=NULL;
-
-
-typedef struct ceentry {
-	int (*fun)(void);
-	struct ceentry *next;
-} ceentry;
-
-static ceentry *cehead=NULL;
-
-
-typedef struct rlentry {
-	void (*fun)(void);
-	struct rlentry *next;
-} rlentry;
-
-static rlentry *rlhead=NULL;
-
-typedef struct pollentry {
-	void (*desc)(std::vector<pollfd>&);
-	void (*serve)(const std::vector<pollfd>&);
-} pollentry;
-
-namespace {
-std::list<pollentry> gPollEntries;
-}
-
-typedef struct eloopentry {
-	void (*fun)(void);
-	struct eloopentry *next;
-} eloopentry;
-
-static eloopentry *eloophead=NULL;
-
 #if defined(LIZARDFS_HAVE_PAM)
 static pam_handle_t *gPAMHandle=NULL;
 #endif
 
-struct timeentry {
-	typedef void (*fun_t)(void);
-	timeentry(uint64_t ne, uint64_t sec, uint64_t off, int mod, fun_t f, bool ms)
-		: nextevent(ne), period(sec), offset(off), mode(mod), fun(f), millisecond_precision(ms) {
-	}
-	uint64_t nextevent;
-	uint64_t period;
-	uint64_t offset;
-	int      mode;
-	fun_t    fun;
-	bool     millisecond_precision;
-};
-
-typedef std::list<timeentry> TimeEntries;
-namespace {
-TimeEntries gTimeEntries;
-}
-
-static std::atomic<uint32_t> now;
-static std::atomic<uint64_t> usecnow;
 static bool gRunAsDaemon = true;
-static std::vector<std::string> gExtraArguments;
-static ExitingStatus gExitingStatus = ExitingStatus::kRunning;
 
 /// When set to true, config will be reloaded after the current loop
-static bool gReloadRequested = false;
 
 static int signalpipe[2];
 
 /* interface */
 
-const std::vector<std::string>& main_get_extra_arguments() {
-	return gExtraArguments;
+static void signal_pipe_desc(std::vector<pollfd> &pdesc) {
+	assert(pdesc.size() == 0);
+	pdesc.push_back({signalpipe[0],POLLIN,0});
 }
 
-bool main_has_extra_argument(std::string name, CaseSensitivity mode) {
-	if (mode == CaseSensitivity::kSensitive) {
-		std::transform(name.begin(), name.end(), name.begin(), tolower);
-	}
-	for (auto option : gExtraArguments) {
-		if (mode == CaseSensitivity::kSensitive) {
-			std::transform(option.begin(), option.end(), option.begin(), tolower);
-		}
-		if (option == name) {
-			return true;
-		}
-	}
-	return false;
-}
-
-void main_make_next_poll_nonblocking() {
-	nextPollNonblocking = true;
-}
-
-void main_destructregister (void (*fun)(void)) {
-	deentry *aux=(deentry*)malloc(sizeof(deentry));
-	passert(aux);
-	aux->fun = fun;
-	aux->next = dehead;
-	dehead = aux;
-}
-
-void main_canexitregister (int (*fun)(void)) {
-	ceentry *aux=(ceentry*)malloc(sizeof(ceentry));
-	passert(aux);
-	aux->fun = fun;
-	aux->next = cehead;
-	cehead = aux;
-}
-
-void main_wantexitregister (void (*fun)(void)) {
-	weentry *aux=(weentry*)malloc(sizeof(weentry));
-	passert(aux);
-	aux->fun = fun;
-	aux->next = wehead;
-	wehead = aux;
-}
-
-void main_reloadregister (void (*fun)(void)) {
-	rlentry *aux=(rlentry*)malloc(sizeof(rlentry));
-	passert(aux);
-	aux->fun = fun;
-	aux->next = rlhead;
-	rlhead = aux;
-}
-
-void main_pollregister (void (*desc)(std::vector<pollfd>&),void (*serve)(const std::vector<pollfd>&)) {
-	gPollEntries.push_back({desc,serve});
-}
-
-void main_eachloopregister (void (*fun)(void)) {
-	eloopentry *aux=(eloopentry*)malloc(sizeof(eloopentry));
-	passert(aux);
-	aux->fun = fun;
-	aux->next = eloophead;
-	eloophead = aux;
-}
-
-void *main_timeregister(int mode, uint64_t seconds, uint64_t offset, void (*fun)(void)) {
-	if (seconds == 0 || offset >= seconds) {
-		return NULL;
-	}
-
-	uint64_t nextevent = ((now + seconds) / seconds) * seconds + offset;
-
-	gTimeEntries.push_front(timeentry(nextevent, seconds, offset, mode, fun, false));
-	return &gTimeEntries.front();
-}
-
-void *main_timeregister_ms(uint64_t period, void (*fun)(void)) {
-	if (period == 0) {
-		return NULL;
-	}
-
-	uint64_t nextevent = usecnow / 1000 + period;
-
-	gTimeEntries.push_front(timeentry(nextevent, period, 0, TIMEMODE_RUN_LATE, fun, true));
-	return &gTimeEntries.front();
-}
-
-void main_timeunregister(void* handler) {
-	for (TimeEntries::iterator it = gTimeEntries.begin(); it != gTimeEntries.end(); ++it) {
-		if (&(*it) == handler) {
-			gTimeEntries.erase(it);
-			return;
-		}
-	}
-	mabort("unregistering unknown handle from time table");
-}
-
-int main_timechange(void* handle, int mode, uint64_t seconds, uint64_t offset) {
-	timeentry *aux = (timeentry*)handle;
-	if (seconds == 0 || offset >= seconds) {
-		return -1;
-	}
-	aux->nextevent = ((now + seconds) / seconds) * seconds + offset;
-	aux->period = seconds;
-	aux->offset = offset;
-	aux->mode = mode;
-	return 0;
-}
-
-int main_timechange_ms(void* handle, uint64_t period) {
-	timeentry *aux = (timeentry*)handle;
-	if (period == 0) {
-		return -1;
-	}
-	aux->nextevent = ((usecnow / 1000 + period) / period) * period;
-	aux->period = period;
-	return 0;
-}
-
-/* internal */
-
-void free_all_registered_entries(void) {
-	deentry *de,*den;
-	ceentry *ce,*cen;
-	weentry *we,*wen;
-	rlentry *re,*ren;
-	eloopentry *ee,*een;
-
-	for (de = dehead ; de ; de = den) {
-		den = de->next;
-		free(de);
-	}
-
-	for (ce = cehead ; ce ; ce = cen) {
-		cen = ce->next;
-		free(ce);
-	}
-
-	for (we = wehead ; we ; we = wen) {
-		wen = we->next;
-		free(we);
-	}
-
-	for (re = rlhead ; re ; re = ren) {
-		ren = re->next;
-		free(re);
-	}
-
-	gPollEntries.clear();
-
-	for (ee = eloophead ; ee ; ee = een) {
-		een = ee->next;
-		free(ee);
-	}
-
-	gTimeEntries.clear();
-}
-
-int canexit() {
-	ceentry *aux;
-	for (aux = cehead ; aux!=NULL ; aux=aux->next) {
-		if (aux->fun()==0) {
-			return 0;
-		}
-	}
-	return 1;
-}
-
-uint32_t main_time() {
-	return now;
-}
-
-uint64_t main_utime() {
-	return usecnow;
-}
-
-uint8_t main_want_to_terminate() {
-	if (gExitingStatus == ExitingStatus::kRunning) {
-		gExitingStatus = ExitingStatus::kWantExit;
-		syslog(LOG_INFO, "Exiting on internal request.");
-		return LIZARDFS_STATUS_OK;
-	} else {
-		syslog(LOG_ERR, "Unable to exit on internal request.");
-		return LIZARDFS_ERROR_NOTPOSSIBLE;
-	}
-}
-
-void main_want_to_reload() {
-	gReloadRequested = true;
-}
-
-void destruct() {
-	deentry *deit;
-	for (deit = dehead ; deit!=NULL ; deit=deit->next) {
-		try {
-			deit->fun();
-		} catch (Exception& ex) {
-			syslog(LOG_WARNING, "term error: %s", ex.what());
-		}
-	}
-}
-
-void mainloop() {
-	uint32_t prevtime  = 0;
-	uint64_t prevmtime = 0;
-	struct timeval tv;
-	eloopentry *eloopit;
-	ceentry *ceit;
-	weentry *weit;
-	rlentry *rlit;
-	std::vector<pollfd> pdesc;
-	int i;
-
-	while (gExitingStatus != ExitingStatus::kDoExit) {
-		pdesc.clear();
-		pdesc.push_back({signalpipe[0],POLLIN,0});
-		for (auto &pollit: gPollEntries) {
-			pollit.desc(pdesc);
-		}
-		i = poll(pdesc.data(),pdesc.size(), nextPollNonblocking ? 0 : 50);
-		nextPollNonblocking = false;
-		gettimeofday(&tv,NULL);
-		usecnow = tv.tv_sec * uint64_t(1000000) + tv.tv_usec;
-		now = tv.tv_sec;
-		if (i<0) {
-			if (errno==EAGAIN) {
-				syslog(LOG_WARNING,"poll returned EAGAIN");
-				usleep(100000);
-				continue;
-			}
-			if (errno!=EINTR) {
-				syslog(LOG_WARNING,"poll error: %s",strerr(errno));
-				break;
-			}
-		} else {
-			if ((pdesc[0].revents)&POLLIN) {
-				uint8_t sigid;
-				if (read(signalpipe[0],&sigid,1)==1) {
-					if (sigid == '\001' && gExitingStatus == ExitingStatus::kRunning) {
-						syslog(LOG_NOTICE,"terminate signal received");
-						gExitingStatus = ExitingStatus::kWantExit;
-					} else if (sigid=='\002') {
-						syslog(LOG_NOTICE,"reloading config files");
-						gReloadRequested = true;
-					} else if (sigid=='\003') {
-						syslog(LOG_NOTICE, "Received SIGUSR1, killing gently...");
-						exit(LIZARDFS_EXIT_STATUS_GENTLY_KILL);
-					}
-				}
-			}
-			for (auto &pollit : gPollEntries) {
-				pollit.serve(pdesc);
-			}
-		}
-		for (eloopit = eloophead ; eloopit != NULL ; eloopit = eloopit->next) {
-			eloopit->fun();
-		}
-
-		uint64_t msecnow = usecnow / 1000;
-
-		if (msecnow < prevmtime) {
-			// time went backward - recalculate next event time
-			for (timeentry& timeit : gTimeEntries) {
-				if (!timeit.millisecond_precision) {
-					continue;
-				}
-
-				uint64_t previous_time_to_run = timeit.nextevent - prevmtime;
-				previous_time_to_run = std::min(previous_time_to_run, timeit.period);
-				timeit.nextevent = msecnow + previous_time_to_run;
-			}
-		}
-
-		if (now<prevtime) {
-			// time went backward !!! - recalculate "nextevent" time
-			// adding previous_time_to_run prevents from running next event too soon.
-			for (timeentry& timeit : gTimeEntries) {
-				if (timeit.millisecond_precision) {
-					continue;
-				}
-
-				uint64_t previous_time_to_run = timeit.nextevent - prevtime;
-				previous_time_to_run = std::min(previous_time_to_run, timeit.period);
-				timeit.nextevent = ((now + previous_time_to_run + timeit.period)
-				                   / timeit.period) * timeit.period + timeit.offset;
-			}
-		} else if (now>prevtime+3600) {
-			// time went forward !!! - just recalculate "nextevent" time
-			for (timeentry& timeit : gTimeEntries) {
-				if (timeit.millisecond_precision) {
-					timeit.nextevent = msecnow + timeit.period;
-					continue;
-				}
-
-				timeit.nextevent = ((now + timeit.period) / timeit.period)
-				                    * timeit.period + timeit.offset;
-			}
-		}
-
-		for (timeentry& timeit : gTimeEntries) {
-			if (timeit.millisecond_precision) {
-				if (msecnow >= timeit.nextevent) {
-					timeit.nextevent = msecnow + timeit.period;
-					timeit.fun();
-				}
-				continue;
-			}
-
-			if (now >= timeit.nextevent) {
-				if (timeit.mode == TIMEMODE_RUN_LATE) {
-					timeit.fun();
-				} else { /* timeit.mode == TIMEMODE_SKIP_LATE */
-					if (now == timeit.nextevent) {
-						timeit.fun();
-					}
-				}
-				timeit.nextevent += ((now - timeit.nextevent + timeit.period)
-				                    / timeit.period) * timeit.period;
-			}
-		}
-		prevtime  = now;
-		prevmtime = usecnow / 1000;
-		if (gExitingStatus == ExitingStatus::kRunning && gReloadRequested) {
-			cfg_reload();
-			for (rlit = rlhead ; rlit!=NULL ; rlit=rlit->next) {
-				try {
-					rlit->fun();
-				} catch (Exception& ex) {
-					syslog(LOG_WARNING, "reload error: %s", ex.what());
-				}
-			}
-			gReloadRequested = false;
-			DEBUG_LOG("main.reload");
-		}
-		if (gExitingStatus == ExitingStatus::kWantExit) {
-			for (weit = wehead ; weit!=NULL ; weit=weit->next) {
-				weit->fun();
-			}
-			gExitingStatus = ExitingStatus::kCanExit;
-		}
-		if (gExitingStatus == ExitingStatus::kCanExit) {
-			i = 1;
-			for (ceit = cehead ; ceit!=NULL && i ; ceit=ceit->next) {
-				if (ceit->fun()==0) {
-					i=0;
-				}
-			}
-			if (i) {
-				gExitingStatus = ExitingStatus::kDoExit;
+static void signal_pipe_serv(const std::vector<pollfd> &pdesc) {
+	if ((pdesc[0].revents)&POLLIN) {
+		uint8_t sigid;
+		if (read(signalpipe[0],&sigid,1)==1) {
+			if (sigid == '\001' && gExitingStatus == ExitingStatus::kRunning) {
+				syslog(LOG_NOTICE,"terminate signal received");
+				gExitingStatus = ExitingStatus::kWantExit;
+			} else if (sigid=='\002') {
+				syslog(LOG_NOTICE,"reloading config files");
+				gReloadRequested = true;
+			} else if (sigid=='\003') {
+				syslog(LOG_NOTICE, "Received SIGUSR1, killing gently...");
+				exit(LIZARDFS_EXIT_STATUS_GENTLY_KILL);
 			}
 		}
 	}
 }
 
+
 int initialize(run_tab* tab) {
 	uint32_t i;
 	int ok;
 	ok = 1;
 	for (i=0 ; (long int)(tab[i].fn)!=0 && ok ; i++) {
-		now = time(NULL);
+		eventloop_updatetime();
 		try {
 			if (tab[i].fn()<0) {
 				lzfs_pretty_syslog(LOG_ERR,"init: %s failed",tab[i].name);
@@ -555,7 +146,7 @@ int initialize(run_tab* tab) {
 			ok = 0;
 		}
 	}
-	now = time(NULL);
+	eventloop_updatetime();
 	return ok;
 }
 
@@ -1172,11 +763,26 @@ void usage(const char *appname) {
 "-u : log undefined config variables\n"
 "-t locktimeout : how long wait for lockfile\n"
 "-c cfgfile : use given config file\n"
+"-p pidfile : write pid to given file\n"
 "-o extra_option : module specific extra option\n"
 	,appname);
 	exit(LIZARDFS_EXIT_STATUS_ERROR);
 }
 
+void makePidFile(const std::string &name) {
+	if (name.empty()) {
+		return;
+	}
+
+	std::ofstream ofs(name, std::ios_base::out | std::ios_base::trunc);
+	if (ofs.fail()) {
+		lzfs_pretty_syslog(LOG_WARNING, "failed to create pid file: %s",
+				name.c_str());
+		return;
+	}
+	ofs << std::to_string(getpid()) << std::endl;
+}
+
 int main(int argc,char **argv) {
 	char *wrkdir;
 	char *appname;
@@ -1188,6 +794,7 @@ int main(int argc,char **argv) {
 	struct rlimit rls;
 	std::string default_cfgfile = ETC_PATH "/" STR(APPNAME) ".cfg";
 	std::string cfgfile = default_cfgfile;
+	std::string pidfile;
 
 	prepareEnvironment();
 	strerr_init();
@@ -1199,7 +806,7 @@ int main(int argc,char **argv) {
 	lockmemory = 0;
 	appname = argv[0];
 
-	while ((ch = getopt(argc, argv, "o:c:dht:uvx?")) != -1) {
+	while ((ch = getopt(argc, argv, "o:c:p:dht:uvx?")) != -1) {
 		switch(ch) {
 			case 'v':
 				printf("version: %s\n",LIZARDFS_PACKAGE_VERSION);
@@ -1216,6 +823,9 @@ int main(int argc,char **argv) {
 			case 'c':
 				cfgfile = optarg;
 				break;
+			case 'p':
+				pidfile = optarg;
+				break;
 			case 'u':
 				logundefined=1;
 				break;
@@ -1259,6 +869,7 @@ int main(int argc,char **argv) {
 		} else {
 			set_signal_handlers(0);
 		}
+		makePidFile(pidfile);
 	}
 
 	if (cfg_load(cfgfile.c_str(), logundefined)==0) {
@@ -1321,6 +932,8 @@ int main(int argc,char **argv) {
 
 	umask(cfg_getuint32("FILE_UMASK",027)&077);
 
+	eventloop_pollregister(signal_pipe_desc, signal_pipe_serv);
+
 	if (!initialize_early()) {
 		if (gRunAsDaemon) {
 			fputc(0, stderr);
@@ -1405,8 +1018,8 @@ int main(int argc,char **argv) {
 			close_msg_channel();
 		}
 		if (initialize_late()) {
-			main_reloadregister(main_reload); // this will be the first thing to do
-			mainloop();
+			eventloop_reloadregister(main_reload); // this will be the first thing to do
+			eventloop_run();
 			ch=LIZARDFS_EXIT_STATUS_SUCCESS;
 		} else {
 			ch=LIZARDFS_EXIT_STATUS_ERROR;
@@ -1418,8 +1031,8 @@ int main(int argc,char **argv) {
 		}
 		ch=LIZARDFS_EXIT_STATUS_ERROR;
 	}
-	destruct();
-	free_all_registered_entries();
+	eventloop_destruct();
+	eventloop_release_resources();
 	signal_cleanup();
 	cfg_term();
 	strerr_term();
