@@ -32,6 +32,8 @@
 
 struct matocsserventry;
 
+extern bool gAvoidSameIpChunkservers;
+
 int chunk_increase_version(uint64_t chunkid);
 int chunk_set_version(uint64_t chunkid,uint32_t version);
 int chunk_change_file(uint64_t chunkid,uint8_t prevgoal,uint8_t newgoal);
@@ -66,7 +68,7 @@ bool chunk_has_only_invalid_copies(uint64_t chunkid);
 
 int chunk_get_fullcopies(uint64_t chunkid,uint8_t *vcopies);
 int chunk_get_partstomodify(uint64_t chunkid, int &recover, int &remove);
-int chunk_repair(uint8_t goal,uint64_t ochunkid,uint32_t *nversion);
+int chunk_repair(uint8_t goal,uint64_t ochunkid,uint32_t *nversion, uint8_t correct_only);
 
 int chunk_getversionandlocations(uint64_t chunkid, uint32_t currentIp, uint32_t& version,
 		uint32_t maxNumberOfChunkCopies, std::vector<ChunkTypeWithAddress>& serversList);
