@@ -1,5 +1,5 @@
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -34,11 +34,12 @@
 #include "common/cwrap.h"
 #include "common/datapack.h"
 #include "common/exceptions.h"
+#include "common/event_loop.h"
 #include "common/goal.h"
 #include "common/main.h"
 #include "common/massert.h"
 #include "common/md5.h"
-#include "common/MFSCommunication.h"
+#include "protocol/MFSCommunication.h"
 #include "common/slogger.h"
 
 typedef struct _exports {
@@ -243,12 +244,12 @@ uint8_t exports_check(uint32_t ip,uint32_t version,uint8_t meta,const uint8_t *p
 	if (f==NULL) {
 		if (nopass) {
 			if (rndstate==0 || rndcode==NULL || passcode==NULL) {
-				return ERROR_NOPASSWORD;
+				return LIZARDFS_ERROR_NOPASSWORD;
 			} else {
-				return ERROR_BADPASSWORD;
+				return LIZARDFS_ERROR_BADPASSWORD;
 			}
 		}
-		return ERROR_EACCES;
+		return LIZARDFS_ERROR_EACCES;
 	}
 	*sesflags = f->sesflags;
 	*rootuid = f->rootuid;
@@ -259,7 +260,7 @@ uint8_t exports_check(uint32_t ip,uint32_t version,uint8_t meta,const uint8_t *p
 	*maxgoal = f->maxgoal;
 	*mintrashtime = f->mintrashtime;
 	*maxtrashtime = f->maxtrashtime;
-	return STATUS_OK;
+	return LIZARDFS_STATUS_OK;
 }
 
 void exports_freelist(exports *arec) {
@@ -420,7 +421,7 @@ int exports_parsegoal(char *goalstr,uint8_t *goal) {
 		// not the whole string was used by strtol
 		return -1;
 	}
-	if (value > UINT8_MAX || !goal::isGoalValid(value)) {
+	if (value > UINT8_MAX || !GoalId::isValid(value)) {
 		return -1;
 	}
 	*goal = value;
@@ -849,8 +850,8 @@ int exports_parseline(char *line,uint32_t lineno,exports *arec) {
 	arec->meta = 0;
 	arec->rootredefined = 0;
 	arec->sesflags = SESFLAG_READONLY;
-	arec->mingoal = goal::kMinGoal;
-	arec->maxgoal = goal::kMaxGoal;
+	arec->mingoal = GoalId::kMin;
+	arec->maxgoal = GoalId::kMax;
 	arec->mintrashtime = 0;
 	arec->maxtrashtime = UINT32_C(0xFFFFFFFF);
 	arec->rootuid = 999;
@@ -953,7 +954,7 @@ void exports_loadexports(void) {
 		if (errno==ENOENT) {
 			throw InitializeException(
 					std::string("exports configuration file (") + ExportsFileName + ") not found"
-					" - please create one (you can copy " ETC_PATH "/mfs/mfsexports.cfg.dist"
+					" - please create one (you can copy " ETC_PATH "/mfsexports.cfg.dist"
 					" to get a base configuration)");
 		} else {
 			throw InitializeException(
@@ -999,29 +1000,10 @@ void exports_loadexports(void) {
 }
 
 void exports_load(void) {
-	int fd;
 	if (ExportsFileName) {
 		free(ExportsFileName);
 	}
-	if (!cfg_isdefined("EXPORTS_FILENAME")) {
-		ExportsFileName = strdup(ETC_PATH "/mfs/mfsexports.cfg");
-		passert(ExportsFileName);
-		if ((fd = open(ExportsFileName,O_RDONLY))<0 && errno==ENOENT) {
-			free(ExportsFileName);
-			ExportsFileName = strdup(ETC_PATH "/mfsexports.cfg");
-			if ((fd = open(ExportsFileName,O_RDONLY))>=0) {
-				lzfs_pretty_syslog(LOG_WARNING,"default sysconf path has changed - please move mfsexports.cfg from " ETC_PATH "/ to " ETC_PATH "/mfs/");
-			} else {
-				free(ExportsFileName);
-				ExportsFileName = strdup(ETC_PATH "/mfs/mfsexports.cfg");
-			}
-		}
-		if (fd>=0) {
-			close(fd);
-		}
-	} else {
-		ExportsFileName = cfg_getstr("EXPORTS_FILENAME", ETC_PATH "/mfs/mfsexports.cfg");
-	}
+	ExportsFileName = cfg_getstr("EXPORTS_FILENAME", ETC_PATH "/mfsexports.cfg");
 	exports_loadexports();
 }
 
@@ -1048,7 +1030,7 @@ int exports_init(void) {
 		// File was empty
 		throw InitializeException(std::string("no exports defined in ") + ExportsFileName);
 	}
-	main_reloadregister(exports_reload);
-	main_destructregister(exports_term);
+	eventloop_reloadregister(exports_reload);
+	eventloop_destructregister(exports_term);
 	return 0;
 }
