@@ -1,6 +1,6 @@
 /*
    Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare,
-   2013-2015 Skytechnology sp. z o.o..
+   2013-2017 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -21,20 +21,16 @@
 
 #include "common/platform.h"
 
-/*! \brief Function processing TaskManager's enqueued tasks.
- *
- * The function processes limited number of tasks in each call,
- * so it should be executed as often as possible.
- */
-void fs_background_task_manager_work();
+#include "common/defective_file_info.h"
 
-/*
- * A function that is called every main loop iteration,
- * recalculates checksums in the backround background using gChecksumBackgroundUpdater
- * Recalculated checksums are FsNodesChecksum, FsEdgesChecksum, XattrChecksum and ChunksChecksum.
- * ChunksChecksum is recalculated externally using chunks_update_checksum_a_bit().
- */
-void fs_background_checksum_recalculation_a_bit();
-void fs_periodic_test_files();
-void fs_periodic_emptytrash(void);
-void fs_periodic_emptyreserved(void);
+std::vector<DefectiveFileInfo> fs_get_defective_nodes_info(uint8_t requested_flags, uint64_t max_entries,
+	                                                   uint64_t &entry_index);
+
+
+void fs_read_periodic_config_file();
+void fs_periodic_master_init();
+
+void fs_test_getdata(uint32_t &loopstart, uint32_t &loopend, uint32_t &files, uint32_t &ugfiles,
+			uint32_t &mfiles, uint32_t &chunks, uint32_t &ugchunks, uint32_t &mchunks,
+			std::string &report);
+void fsnodes_periodic_remove(uint32_t inode);
