@@ -300,6 +300,8 @@ int mainloop(struct fuse_args *args,const char* mp,int mt,int fg) {
 				gMountOptions.chunkserverconnectreadto,
 				gMountOptions.chunkserverwavereadto,
 				gMountOptions.chunkservertotalreadto,
+				gMountOptions.cacheexpirationtime,
+				gMountOptions.readaheadmaxwindowsize,
 				gMountOptions.prefetchxorstripes,
 				gMountOptions.bandwidthoveruse);
 		write_data_init(gMountOptions.writecachesize,
@@ -339,6 +341,7 @@ int mainloop(struct fuse_args *args,const char* mp,int mt,int fg) {
 				gMountOptions.debug,
 				gMountOptions.keepcache,
 				gMountOptions.direntrycacheto,
+				gMountOptions.direntrycachesize,
 				gMountOptions.entrycacheto,
 				gMountOptions.attrcacheto,
 				gMountOptions.mkdircopysgid,
@@ -601,6 +604,7 @@ int main(int argc, char *argv[]) try {
 			|| strcasecmp(gMountOptions.cachemode,"NONE")==0
 			|| strcasecmp(gMountOptions.cachemode,"NEVER")==0) {
 		gMountOptions.keepcache=2;
+		gMountOptions.cacheexpirationtime=0;
 	} else {
 		fprintf(stderr,"unrecognized cachemode option\nsee: %s -h for help\n",argv[0]);
 		return 1;
@@ -682,6 +686,11 @@ int main(int argc, char *argv[]) try {
 				gMountOptions.aclcachesize);
 		gMountOptions.aclcachesize = 1000 * 1000;
 	}
+	if (gMountOptions.direntrycachesize > 10000000) {
+		fprintf(stderr,"directory entry cache size too big (%u) - decreased to 10000000\n",
+				gMountOptions.direntrycachesize);
+		gMountOptions.direntrycachesize = 10000000;
+	}
 
 	make_fsname(&args);
 
