@@ -1,18 +1,40 @@
+/*
+   Copyright 2013-2014 EditShare, 2013-2017 Skytechnology sp. z o.o.
+
+   This file is part of LizardFS.
+
+   LizardFS is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, version 3.
+
+   LizardFS is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with LizardFS. If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "common/platform.h"
 
 #include <fcntl.h>
 #include <signal.h>
+#include <stdio.h>
 #include <iostream>
 #include <boost/make_shared.hpp>
 #include <polonaise/polonaise_constants.h>
 #include <polonaise/Polonaise.h>
 #include <thrift/protocol/TBinaryProtocol.h>
 #include <thrift/server/TThreadedServer.h>
-#include <thrift/transport/TServerSocket.h>
 #include <thrift/transport/TBufferTransports.h>
+#include <thrift/transport/TPipeServer.h>
+#include <thrift/transport/TServerSocket.h>
 
 #include "common/crc.h"
-#include "mount/csdb.h"
+#include "common/slogger.h"
+#include "common/sockets.h"
+#include "mount/errno_defs.h"
 #include "mount/g_io_limiters.h"
 #include "mount/lizard_client.h"
 #include "mount/mastercomm.h"
@@ -23,6 +45,8 @@
 #include "mount/polonaise/options.h"
 #include "mount/polonaise/setup.h"
 
+#include "mount/stat_defs.h" // !!! This must be the last include. Do not move !!!
+
 using namespace ::polonaise;
 
 /**
@@ -57,7 +81,7 @@ static Failure makeFailure(std::string message) {
  * \throw Failure when errno number is unknown
  * \return corresponding StatusCode for Polonaise client
  */
-StatusCode::type toStatusCode(int errNo) throw(Failure) {
+StatusCode::type toStatusCode(int errNo) {
 	static std::map<int, StatusCode::type> statuses = {
 			{ E2BIG, StatusCode::kE2BIG },
 			{ EACCES, StatusCode::kEACCES },
@@ -177,7 +201,7 @@ static int toLizardFsFlags(int32_t polonaiseFlags) {
  * \throw Failure when context couldn't be converted
  * \return converted context
  */
-static LizardClient::Context toLizardFsContext(const Context& ctx) throw(Failure) {
+static LizardClient::Context toLizardFsContext(const Context& ctx) {
 	try {
 		LizardClient::Context outCtx(toUint32(ctx.uid), toUint32(ctx.gid), toInt32(ctx.pid),
 				toUint32(ctx.umask));
@@ -193,7 +217,7 @@ static LizardClient::Context toLizardFsContext(const Context& ctx) throw(Failure
  * \throw Failure when file type is unknown
  * \return file mode with a file type filled
  */
-static mode_t toModeFileType(const FileType::type type) throw(Failure) {
+static mode_t toModeFileType(const FileType::type type) {
 	mode_t result;
 	switch (type) {
 		case FileType::kDirectory:
@@ -230,7 +254,7 @@ static mode_t toModeFileType(const FileType::type type) throw(Failure) {
  * \throw Failure when file type is incorrect
  * \return Unix file mode
  */
-static mode_t toLizardFsMode(const FileType::type& type, const Mode& mode) throw(Failure) {
+static mode_t toLizardFsMode(const FileType::type& type, const Mode& mode) {
 	mode_t result = toModeFileType(type);
 	result |= mode.setUid ? S_ISUID : 0;
 	result |= mode.setGid ? S_ISGID : 0;
@@ -273,7 +297,7 @@ static int toLizardfsAttributesSet(int32_t set) {
  * \throw Failure when file type is unknown
  * \return result stats
  */
-static struct stat toStructStat(const FileStat& fstat) throw(Failure) {
+static struct stat toStructStat(const FileStat& fstat) {
 	struct stat result;
 	result.st_mode = toLizardFsMode(fstat.type, fstat.mode);
 	result.st_dev = fstat.dev;
@@ -283,8 +307,12 @@ static struct stat toStructStat(const FileStat& fstat) throw(Failure) {
 	result.st_gid = fstat.gid;
 	result.st_rdev = fstat.rdev;
 	result.st_size = fstat.size;
+#ifdef LIZARDFS_HAVE_STRUCT_STAT_ST_BLKSIZE
 	result.st_blksize = fstat.blockSize;
+#endif
+#ifdef LIZARDFS_HAVE_STRUCT_STAT_ST_BLOCKS
 	result.st_blocks = fstat.blocks;
+#endif
 	result.st_atime = fstat.atime;
 	result.st_mtime = fstat.mtime;
 	result.st_ctime = fstat.ctime;
@@ -297,7 +325,7 @@ static struct stat toStructStat(const FileStat& fstat) throw(Failure) {
  * \throw Failure when file type is unknown
  * \return Polonaise files stats
  */
-static FileStat toFileStat(const struct stat& in) throw(Failure) {
+static FileStat toFileStat(const struct stat& in) {
 	FileStat reply;
 	switch (in.st_mode & S_IFMT) {
 		case S_IFDIR:
@@ -345,8 +373,12 @@ static FileStat toFileStat(const struct stat& in) throw(Failure) {
 	reply.gid = in.st_gid;
 	reply.rdev = in.st_rdev;
 	reply.size = in.st_size;
+#ifdef LIZARDFS_HAVE_STRUCT_STAT_ST_BLKSIZE
 	reply.blockSize = in.st_blksize;
+#endif
+#ifdef LIZARDFS_HAVE_STRUCT_STAT_ST_BLOCKS
 	reply.blocks = in.st_blocks;
+#endif
 	reply.atime = in.st_atime;
 	reply.mtime = in.st_mtime;
 	reply.ctime = in.st_ctime;
@@ -359,7 +391,7 @@ static FileStat toFileStat(const struct stat& in) throw(Failure) {
  * \throw Failure when file type couldn't be converted
  * \return Polonaise's entry
  */
-static EntryReply toEntryReply(const LizardClient::EntryParam& in) throw(Failure) {
+static EntryReply toEntryReply(const LizardClient::EntryParam& in) {
 	if (in.attr_timeout < 0.0 || in.entry_timeout < 0.0) {
 		throw makeFailure("Invalid timeout");
 	}
@@ -386,6 +418,7 @@ static EntryReply toEntryReply(const LizardClient::EntryParam& in) throw(Failure
 			throw makeStatus(toStatusCode(ex.errNo));\
 		} catch (Failure& ex) {\
 			std::cerr << __FUNCTION__ << " failure: " << ex.message << std::endl;\
+			lzfs_pretty_syslog(LOG_ERR, "%s Failure: %s", __FUNCTION__, ex.message.c_str());\
 			throw makeFailure(std::string(__FUNCTION__) + ": " + ex.message);\
 		}
 
@@ -509,7 +542,7 @@ public:
 	 * Implement Polonaise.readdir method
 	 * \note for more information, see the protocol definition in Polonaise sources
 	 */
-	void readdir(std::vector<DirectoryEntry> & _return, const Context& context,
+	void readdir(std::vector<polonaise::DirectoryEntry> & _return, const Context& context,
 			const Inode inode, const int64_t firstEntryOffset,
 			const int64_t maxNumberOfEntries, const Descriptor descriptor) {
 		OPERATION_PROLOG
@@ -625,13 +658,29 @@ public:
 		if (descriptor == g_polonaise_constants.kNullDescriptor) {
 			throw makeFailure("Null descriptor");
 		}
-		std::vector<uint8_t> buffer = LizardClient::read(
-				toLizardFsContext(context),
-				toUint64(inode),
-				size,
-				offset,
-				getFileInfo(descriptor));
-		_return.assign(reinterpret_cast<const char*>(buffer.data()), buffer.size());
+
+		if (LizardClient::isSpecialInode(inode)) {
+			std::vector<uint8_t> buffer = LizardClient::read_special_inode(
+					toLizardFsContext(context),
+					toUint64(inode),
+					size,
+					offset,
+					getFileInfo(descriptor));
+			_return.assign(reinterpret_cast<const char*>(buffer.data()), buffer.size());
+		} else {
+			auto result = LizardClient::read(
+					toLizardFsContext(context),
+					toUint64(inode),
+					size,
+					offset,
+					getFileInfo(descriptor));
+			small_vector<std::pair<void *, std::size_t>, 8> reply;
+			result.toIoVec(reply, offset, size);
+			for (const auto &iov : reply) {
+				const char *base = (const char *)iov.first;
+				_return.append(base, base + iov.second);
+			}
+		}
 		OPERATION_EPILOG
 	}
 
@@ -787,6 +836,62 @@ public:
 		OPERATION_EPILOG
 	}
 
+	/**
+	 * Implement Polonaise.getxattr method
+	 * \note for more information, see the protocol definition in Polonaise sources
+	 */
+	void getxattr(std::string& _return, const Context& context, const Inode inode,
+		      const std::string& name, const int64_t size) {
+		OPERATION_PROLOG
+		uint32_t position = 0;
+		auto a = LizardClient::getxattr(toLizardFsContext(context), toUint64(inode),
+						name.c_str(), size, position);
+		if (size == 0) {
+			a = LizardClient::getxattr(toLizardFsContext(context), toUint64(inode),
+						   name.c_str(),  a.valueLength, position);
+		}
+		_return.assign(a.valueBuffer.begin(), a.valueBuffer.end());
+		OPERATION_EPILOG
+	}
+
+	/**
+	 * Implement Polonaise.setxattr method
+	 * \note for more information, see the protocol definition in Polonaise sources
+	 */
+	void setxattr(const Context& context, const Inode inode, const std::string& name,
+		      const std::string& value, const int64_t size, const int32_t flags) {
+		OPERATION_PROLOG
+		uint32_t position = 0;
+		LizardClient::setxattr(toLizardFsContext(context), toUint64(inode), name.c_str(), value.c_str(),
+				       size, flags, position);
+		OPERATION_EPILOG
+	}
+
+	/**
+	 * Implement Polonaise.listxattr method
+	 * \note for more information, see the protocol definition in Polonaise sources
+	 */
+	void listxattr(std::string& _return, const Context& context, const Inode inode,
+		       const int64_t size) {
+		OPERATION_PROLOG
+		auto a = LizardClient::listxattr(toLizardFsContext(context), toUint64(inode), size);
+		if (size == 0) {
+			a = LizardClient::listxattr(toLizardFsContext(context), toUint64(inode), a.valueLength);
+		}
+		_return.assign(a.valueBuffer.begin(), a.valueBuffer.end());
+		OPERATION_EPILOG
+	}
+
+	/**
+	 * Implement Polonaise.removexattr method
+	 * \note for more information, see the protocol definition in Polonaise sources
+	 */
+	void removexattr(const Context& context, const Inode inode, const std::string& name) {
+		OPERATION_PROLOG
+		LizardClient::removexattr(toLizardFsContext(context), toUint64(inode), name.c_str());
+		OPERATION_EPILOG
+	}
+
 private:
 	/**
 	 * Create a new entry of opened file
@@ -797,7 +902,7 @@ private:
 		std::lock_guard<std::mutex> lock(mutex_);
 		Descriptor descriptor = ++lastDescriptor_;
 		fileInfos_.insert({descriptor,
-				LizardClient::FileInfo(toLizardFsFlags(polonaiseFlags), 0, 0, descriptor)});
+				LizardClient::FileInfo(toLizardFsFlags(polonaiseFlags), 0, 0, descriptor, 0)});
 		return descriptor;
 	}
 
@@ -874,9 +979,78 @@ void termhandle(int) {
 	}
 }
 
+#ifndef _WIN32
+bool daemonize() {
+	pid_t pid;
+
+	pid = fork();
+	if (pid) {
+		if (pid > 0) {
+			exit(0);
+		}
+
+		lzfs_pretty_syslog(LOG_ERR, "First fork failed: %s", strerror(errno));
+		return false ;
+	}
+
+	/* setsid() return value is ignored, because behaviour is the same regardless of
+	 * its result.
+	 */
+	setsid();
+	int r = chdir("/");
+	if (r < 0) {
+		lzfs_pretty_syslog(LOG_ERR, "Change directory failed: %s", strerror(errno));
+	}
+	umask(0);
+
+	pid = fork();
+	if (pid) {
+		if (pid > 0) {
+			exit(0);
+		}
+
+		lzfs_pretty_syslog(LOG_ERR, "Second fork failed: %s", strerror(errno));
+		return false;
+	}
+
+	close(0);
+	close(1);
+	close(2);
+
+	if (open("/dev/null", O_RDONLY) < 0) {
+		lzfs_pretty_syslog(LOG_ERR, "Unable to open /dev/null: %s", strerror(errno));
+		return false;
+	}
+	if (open("/dev/null", O_WRONLY) < 0) {
+		lzfs_pretty_syslog(LOG_ERR, "Unable to open /dev/null: %s", strerror(errno));
+		return false;
+	}
+	if (dup(1) < 0) {
+		lzfs_pretty_syslog(LOG_ERR, "Unable to duplicate stdout descriptor: %s", strerror(errno));
+		return false;
+	}
+
+	return true;
+}
+#endif
+
 int main (int argc, char **argv) {
+	bool userwlock = true;
+	auto writeworkers = 30;
+	auto writewindowsize = 15;
+	auto chunkserverrtt = 200;
+	auto chunkserverconnectreadto = 2000;
+	auto chunkserverwavereadto = 500;
+	auto chunkservertotalreadto = 2000;
+	auto cacheexpirationtime = 500;
+	auto readaheadmaxwindowsize = 4096;
+	bool prefetchFullXorStripes = true;
+	auto bandwidthOveruse = 1.25;
+	auto chunkserverwriteto = 5000;
+	auto cacheperinodepercentage = 25;
 	parse_command_line(argc, argv, gSetup);
 
+#ifndef _WIN32
 	struct sigaction sa;
 	sa.sa_flags = 0;
 	sigemptyset(&sa.sa_mask);
@@ -887,7 +1061,20 @@ int main (int argc, char **argv) {
 	sigaction(SIGUSR1, &sa, nullptr);
 #endif
 
+	openlog("lizardfs-polonaise-server", 0, LOG_DAEMON);
+
+	// Daemonize if needed
+	if (gSetup.make_daemon && !daemonize()) {
+		lzfs_pretty_syslog(LOG_ERR, "Unable to daemonize lizardfs-polonaise-server");
+		return EXIT_FAILURE;
+	}
+#endif
+
+	setvbuf(stdout, NULL, _IONBF, 0);
+	setvbuf(stderr, NULL, _IONBF, 0);
+
 	// Initialize LizardFS client
+	socketinit();
 	strerr_init();
 	mycrc32_init();
 	if (fs_init_master_connection(nullptr,
@@ -895,6 +1082,7 @@ int main (int argc, char **argv) {
 				0, gSetup.mountpoint.c_str(), gSetup.subfolder.c_str(), nullptr,
 				gSetup.forget_password, 0, gSetup.io_retries, gSetup.report_reserved_period) < 0) {
 		std::cerr << "Can't initialize connection with master server" << std::endl;
+		lzfs_pretty_syslog(LOG_ERR, "Can't initialize connection with master server");
 		return 2;
 	}
 	symlink_cache_init();
@@ -904,12 +1092,20 @@ int main (int argc, char **argv) {
 	gLocalIoLimiter();
 	IoLimitsConfigLoader loader;
 	gMountLimiter().loadConfiguration(loader);
-	csdb_init();
-	read_data_init(gSetup.io_retries);
-	write_data_init(gSetup.write_buffer_size * 1024 * 1024, gSetup.io_retries);
-	LizardClient::init(gSetup.debug, true, gSetup.direntry_cache_timeout,
+	read_data_init(gSetup.io_retries,
+			chunkserverrtt,
+			chunkserverconnectreadto,
+			chunkserverwavereadto,
+			chunkservertotalreadto,
+			cacheexpirationtime,
+			readaheadmaxwindowsize,
+			prefetchFullXorStripes,
+			bandwidthOveruse);
+	write_data_init(gSetup.write_buffer_size, gSetup.io_retries, writeworkers,
+			writewindowsize, chunkserverwriteto, cacheperinodepercentage);
+	LizardClient::init(gSetup.debug, true, gSetup.direntry_cache_timeout, gSetup.direntry_cache_size,
 			gSetup.entry_cache_timeout, gSetup.attr_cache_timeout,
-			!gSetup.no_mkdir_copy_sgid, gSetup.sugid_clear_mode, false, 0, 0);
+			!gSetup.no_mkdir_copy_sgid, gSetup.sugid_clear_mode, gSetup.enable_acl, userwlock, 0, 0);
 
 	// Thrift server start
 	using namespace ::apache::thrift;
@@ -917,21 +1113,33 @@ int main (int argc, char **argv) {
 	using namespace ::apache::thrift::server;
 	boost::shared_ptr<PolonaiseHandler> handler(new PolonaiseHandler());
 	boost::shared_ptr<TProcessor> processor(new PolonaiseProcessor(handler));
-	boost::shared_ptr<TServerTransport> serverTransport(new TServerSocket(gSetup.bind_port));
 	boost::shared_ptr<TTransportFactory> transportFactory(new BigBufferedTransportFactory());
 	boost::shared_ptr<TProtocolFactory> protocolFactory(new TBinaryProtocolFactory());
-	gServer.reset(
-			new TThreadedServer(processor, serverTransport, transportFactory, protocolFactory));
+	boost::shared_ptr<TServerTransport> serverTransport;
+
+#ifdef _WIN32
+	if (gSetup.bind_port > 0) {
+		serverTransport.reset(new TServerSocket(gSetup.bind_port));
+	} else {
+		static const int kPipeBufferSize = 128 * 1024;
+		serverTransport.reset(new TPipeServer(gSetup.pipe_name, kPipeBufferSize));
+	}
+#else
+	serverTransport.reset(new TServerSocket(gSetup.bind_port));
+#endif
+
+	gServer.reset(new TThreadedServer(processor, serverTransport, transportFactory, protocolFactory));
+
 	if (gTerminated == 0) {
 		gServer->serve();
 	}
 
 	write_data_term();
 	read_data_term();
-	csdb_term();
 	masterproxy_term();
 	fs_term();
 	symlink_cache_term();
+	socketrelease();
 
 	return 0;
 }
