@@ -23,6 +23,11 @@ setup_local_empty_lizardfs() {
 	lizardfs_info_[chunkserver_count]=$number_of_chunkservers
 	lizardfs_info_[admin_password]=${ADMIN_PASSWORD:-password}
 
+	# Try to enable core dumps if possible
+	if [[ $(ulimit -c) == 0 ]]; then
+		ulimit -c unlimited || ulimit -c 100000000 || ulimit -c 1000000 || ulimit -c 10000 || :
+	fi
+
 	# Prepare directories for LizardFS
 	mkdir -p "$etcdir" "$vardir"
 
@@ -466,12 +471,17 @@ create_mfsmount_cfg_() {
 
 do_mount_() {
 	local mount_id=$1
+	local workingdir=$TEMP_DIR/var/mount$mount_id
+	# Make sure mount process is in a writable directory (for core dumps)
+	mkdir -p $workingdir
+	cd $workingdir
 	for try in $(seq 1 $max_tries); do
 		${lizardfs_info_[mntcall${mount_id}]} && return 0
 		echo "Retrying in 1 second ($try/$max_tries)..."
 		sleep 1
 	done
 	echo "Cannot mount in $mount_dir, exiting"
+	cd -
 	exit 2
 }
 
